@@ -6,17 +6,17 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2004 Insecure.Com LLC. Nmap       *
- * is also a registered trademark of Insecure.Com LLC.  This program is    *
- * free software; you may redistribute and/or modify it under the          *
- * terms of the GNU General Public License as published by the Free        *
- * Software Foundation; Version 2.  This guarantees your right to use,     *
- * modify, and redistribute this software under certain conditions.  If    *
- * you wish to embed Nmap technology into proprietary software, we may be  *
- * willing to sell alternative licenses (contact sales@insecure.com).      *
- * Many security scanner vendors already license Nmap technology such as  *
- * our remote OS fingerprinting database and code, service/version         *
- * detection system, and port scanning code.                               *
+ * The Nmap Security Scanner is (C) 1996-2008 Insecure.Com LLC. Nmap is    *
+ * also a registered trademark of Insecure.Com LLC.  This program is free  *
+ * software; you may redistribute and/or modify it under the terms of the  *
+ * GNU General Public License as published by the Free Software            *
+ * Foundation; Version 2 with the clarifications and exceptions described  *
+ * below.  This guarantees your right to use, modify, and redistribute     *
+ * this software under certain conditions.  If you wish to embed Nmap      *
+ * technology into proprietary software, we sell alternative licenses      *
+ * (contact sales@insecure.com).  Dozens of software vendors already       *
+ * license Nmap technology such as host discovery, port scanning, OS       *
+ * detection, and version detection.                                       *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
@@ -26,8 +26,11 @@
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
  *   nmap-os-fingerprints or nmap-service-probes.                          *
- * o Executes Nmap                                                         *
- * o Integrates/includes/aggregates Nmap into an executable installer      *
+ * o Executes Nmap and parses the results (as opposed to typical shell or  *
+ *   execution-menu apps, which simply display raw Nmap output and so are  *
+ *   not derivative works.)                                                * 
+ * o Integrates/includes/aggregates Nmap into a proprietary executable     *
+ *   installer, such as those produced by InstallShield.                   *
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
@@ -36,7 +39,7 @@
  * These restrictions only apply when you actually redistribute Nmap.  For *
  * example, nothing stops you from writing and selling a proprietary       *
  * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://www.insecure.org/nmap/ to download Nmap.                         *
+ * http://nmap.org to download Nmap.                                       *
  *                                                                         *
  * We don't consider these to be added restrictions on top of the GPL, but *
  * just a clarification of how we interpret "derived works" as it applies  *
@@ -48,14 +51,23 @@
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to many    *
- * security vendors, and generally include a perpetual license as well as  *
- * providing for priority support and updates as well as helping to fund   *
- * the continued development of Nmap technology.  Please email             *
+ * applications and appliances.  These contracts have been sold to dozens  *
+ * of software vendors, and generally include a perpetual license as well  *
+ * as providing for priority support and updates as well as helping to     *
+ * fund the continued development of Nmap technology.  Please email        *
  * sales@insecure.com for further information.                             *
  *                                                                         *
+ * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+ * permission to link the code of this program with any version of the     *
+ * OpenSSL library which is distributed under a license identical to that  *
+ * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * combinations including the two. You must obey the GNU GPL in all        *
+ * respects for all of the code used other than OpenSSL.  If you modify    *
+ * this file, you may extend this exception to your version of the file,   *
+ * but you are not obligated to do so.                                     *
+ *                                                                         *
  * If you received these files with a written license agreement or         *
- * contract stating terms other than the (GPL) terms above, then that      *
+ * contract stating terms other than the terms above, then that            *
  * alternative license agreement takes precedence over these comments.     *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
@@ -66,7 +78,7 @@
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
  * to fyodor@insecure.org for possible incorporation into the main         *
- * distribution.  By sending these changes to Fyodor or one the            *
+ * distribution.  By sending these changes to Fyodor or one of the         *
  * Insecure.Org development mailing lists, it is assumed that you are      *
  * offering Fyodor and Insecure.Com LLC the unlimited, non-exclusive right *
  * to reuse, modify, and relicense the code.  Nmap will always be          *
@@ -81,15 +93,25 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html .                                  *
+ * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
+ * with Nmap.                                                              *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: Target.cc,v 1.16 2004/04/19 02:02:23 fyodor Exp $ */
+/* $Id: Target.cc 7145 2008-04-11 03:54:44Z david $ */
+
+#ifdef WIN32
+#include "nmap_winconfig.h"
+#endif
 
 #include "Target.h"
-#include "osscan.h"
+#include <dnet.h>
 #include "nbase.h"
+#include "NmapOps.h"
+#include "utils.h"
+#include "nmap_error.h"
+
+extern NmapOps o;
 
 Target::Target() {
   Initialize();
@@ -97,22 +119,39 @@ Target::Target() {
 
 void Target::Initialize() {
   hostname = NULL;
+  targetname = NULL;
   memset(&seq, 0, sizeof(seq));
+  distance = -1;
   FPR = NULL;
-  osscan_performed = 0;
+  osscan_flag = OS_NOTPERF;
   wierd_responses = flags = 0;
   memset(&to, 0, sizeof(to));
-  memset(&host_timeout, 0, sizeof(host_timeout));
-  memset(&firewallmode, 0, sizeof(struct firewallmodeinfo));
-  timedout = 0;
-  device[0] = '\0';
   memset(&targetsock, 0, sizeof(targetsock));
   memset(&sourcesock, 0, sizeof(sourcesock));
-  targetsocklen = sourcesocklen = 0;
+  memset(&nexthopsock, 0, sizeof(nexthopsock));
+  targetsocklen = sourcesocklen = nexthopsocklen = 0;
+  directly_connected = -1;
   targetipstring[0] = '\0';
   nameIPBuf = NULL;
   memset(&MACaddress, 0, sizeof(MACaddress));
-  MACaddress_set = false;
+  memset(&SrcMACaddress, 0, sizeof(SrcMACaddress));
+  memset(&NextHopMACaddress, 0, sizeof(NextHopMACaddress));
+  MACaddress_set = SrcMACaddress_set = NextHopMACaddress_set = false;
+  htn.msecs_used = 0;
+  htn.toclock_running = false;
+  interface_type = devt_other;
+	devname[0] = '\0';
+	devfullname[0] = '\0';
+  state_reason_init(&reason);
+}
+
+
+const char * Target::deviceName() { 
+	return (devname[0] != '\0')? devname : NULL;
+}
+
+const char * Target::deviceFullName() { 
+	return (devfullname[0] != '\0')? devfullname : NULL; 
 }
 
 void Target::Recycle() {
@@ -130,13 +169,15 @@ void Target::FreeInternal() {
   if (hostname)
     free(hostname);
 
+  if (targetname)
+    free(targetname);
+
   if (nameIPBuf) {
     free(nameIPBuf);
     nameIPBuf = NULL;
   }
 
   if (FPR) delete FPR;
-
 }
 
 /*  Creates a "presentation" formatted string out of the IPv4/IPv6 address.
@@ -160,7 +201,8 @@ void Target::GenerateIPString() {
 /* Fills a sockaddr_storage with the AF_INET or AF_INET6 address
      information of the target.  This is a preferred way to get the
      address since it is portable for IPv6 hosts.  Returns 0 for
-     success. */
+     success. ss_len must be provided.  It is not examined, but is set
+     to the size of the sockaddr copied in. */
 int Target::TargetSockAddr(struct sockaddr_storage *ss, size_t *ss_len) {
   assert(ss);
   assert(ss_len);  
@@ -181,10 +223,13 @@ void Target::setTargetSockAddr(struct sockaddr_storage *ss, size_t ss_len) {
     /* We had an old target sock, so we better blow away the hostname as
        this one may be new. */
     setHostName(NULL);
+    setTargetName(NULL);
   }
   memcpy(&targetsock, ss, ss_len);
   targetsocklen = ss_len;
   GenerateIPString();
+  /* The ports array needs to know a name too */
+  ports.setIdStr(targetipstr());
 }
 
 // Returns IPv4 host address or {0} if unavailable.
@@ -253,11 +298,9 @@ void Target::setHostName(char *name) {
     hostname = NULL;
   }
   if (name) {
-    if (strchr(name, '%')) {
-    }
     p = hostname = strdup(name);
     while (*p) {
-      // I think only a-z A-Z 0-9 . and - are allowed, but I'l be a little more
+      // I think only a-z A-Z 0-9 . and - are allowed, but I'll be a little more
       // generous.
       if (!isalnum(*p) && !strchr(".-+=:_~*", *p)) {
 	log_write(LOG_STDOUT, "Illegal character(s) in hostname -- replacing with '*'\n");
@@ -268,16 +311,26 @@ void Target::setHostName(char *name) {
   }
 }
 
- /* Generates the a printable string consisting of the host's IP
+void Target::setTargetName(char *name) {
+  if (targetname) {
+    free(targetname);
+    targetname = NULL;
+  }
+  if (name) {
+    targetname = strdup(name);
+  }
+}
+
+ /* Generates a printable string consisting of the host's IP
      address and hostname (if available).  Eg "www.insecure.org
      (64.71.184.53)" or "fe80::202:e3ff:fe14:1102".  The name is
      written into the buffer provided, which is also returned.  Results
-     that do not fit in bufflen will be truncated. */
+     that do not fit in buflen will be truncated. */
 const char *Target::NameIP(char *buf, size_t buflen) {
   assert(buf);
   assert(buflen > 8);
   if (hostname) {
-    snprintf(buf, buflen, "%s (%s)", hostname, targetipstring);
+    Snprintf(buf, buflen, "%s (%s)", hostname, targetipstring);
   } else Strncpy(buf, targetipstring, buflen);
   return buf;
 }
@@ -288,6 +341,83 @@ const char *Target::NameIP() {
   return NameIP(nameIPBuf, MAXHOSTNAMELEN + INET6_ADDRSTRLEN);
 }
 
+  /* Returns the next hop for sending packets to this host.  Returns true if
+     next_hop was filled in.  It might be false, for example, if
+     next_hop has never been set */
+bool Target::nextHop(struct sockaddr_storage *next_hop, size_t *next_hop_len) {
+  if (nexthopsocklen <= 0)
+    return false;
+  assert(nexthopsocklen <= sizeof(*next_hop));
+  if (next_hop)
+    memcpy(next_hop, &nexthopsock, nexthopsocklen);
+  if (next_hop_len)
+    *next_hop_len = nexthopsocklen;
+  return true;
+}
+
+  /* If the host is directly connected on a network, set and retrieve
+     that information here.  directlyConnected() will abort if it hasn't
+     been set yet.  */
+void Target::setDirectlyConnected(bool connected) {
+  directly_connected = connected? 1 : 0;
+}
+
+int Target::directlyConnectedOrUnset(){
+    return directly_connected;
+}
+
+bool Target::directlyConnected() {
+  assert(directly_connected == 0 || directly_connected == 1);
+  return directly_connected;
+}
+
+/* Note that it is OK to pass in a sockaddr_in or sockaddr_in6 casted
+     to sockaddr_storage */
+void Target::setNextHop(struct sockaddr_storage *next_hop, size_t next_hop_len) {
+  assert(next_hop_len > 0 && next_hop_len <= sizeof(nexthopsock));
+  memcpy(&nexthopsock, next_hop, next_hop_len);
+  nexthopsocklen = next_hop_len;
+}
+
+
+  /* Starts the timeout clock for the host running (e.g. you are
+     beginning a scan).  If you do not have the current time handy,
+     you can pass in NULL.  When done, call stopTimeOutClock (it will
+     also automatically be stopped of timedOut() returns true) */
+void Target::startTimeOutClock(const struct timeval *now) {
+  assert(htn.toclock_running == false);
+  htn.toclock_running = true;
+  if (now) htn.toclock_start = *now;
+  else gettimeofday(&htn.toclock_start, NULL);
+}
+  /* The complement to startTimeOutClock. */
+void Target::stopTimeOutClock(const struct timeval *now) {
+  struct timeval tv;
+  assert(htn.toclock_running == true);
+  htn.toclock_running = false;
+  if (now) tv = *now;
+  else gettimeofday(&tv, NULL);
+  htn.msecs_used += TIMEVAL_MSEC_SUBTRACT(tv, htn.toclock_start);
+}
+  /* Returns whether the host is timedout.  If the timeoutclock is
+     running, counts elapsed time for that.  Pass NULL if you don't have the
+     current time handy.  You might as well also pass NULL if the
+     clock is not running, as the func won't need the time. */
+bool Target::timedOut(const struct timeval *now) {
+  unsigned long used = htn.msecs_used;
+  struct timeval tv;
+
+  if (!o.host_timeout) return false;
+  if (htn.toclock_running) {
+    if (now) tv = *now;
+    else gettimeofday(&tv, NULL);
+    used += TIMEVAL_MSEC_SUBTRACT(tv, htn.toclock_start);
+  }
+
+  return (used > o.host_timeout)? true : false;
+}
+
+
 /* Returns zero if MAC address set successfully */
 int Target::setMACAddress(const u8 *addy) {
   if (!addy) return 1;
@@ -296,7 +426,50 @@ int Target::setMACAddress(const u8 *addy) {
   return 0;
 }
 
+int Target::setSrcMACAddress(const u8 *addy) {
+  if (!addy) return 1;
+  memcpy(SrcMACaddress, addy, 6);
+  SrcMACaddress_set = 1;
+  return 0;
+}
+
+int Target::setNextHopMACAddress(const u8 *addy) {
+  if (!addy) return 1;
+  memcpy(NextHopMACaddress, addy, 6);
+  NextHopMACaddress_set = 1;
+  return 0;
+}
+
+/* Set the device names so that they can be returned by deviceName()
+   and deviceFullName().  The normal name may not include alias
+   qualifier, while the full name may include it (e.g. "eth1:1").  If
+   these are non-null, they will overwrite the stored version */
+void Target::setDeviceNames(const char *name, const char *fullname) {
+  if (name) Strncpy(devname, name, sizeof(devname));
+  if (fullname) Strncpy(devfullname, fullname, sizeof(devfullname));
+}
+
 /* Returns the 6-byte long MAC address, or NULL if none has been set */
 const u8 *Target::MACAddress() {
   return (MACaddress_set)? MACaddress : NULL;
 }
+
+const u8 *Target::SrcMACAddress() {
+  return (SrcMACaddress_set)? SrcMACaddress : NULL;
+}
+
+const u8 *Target::NextHopMACAddress() {
+  return (NextHopMACaddress_set)? NextHopMACaddress : NULL;
+}
+
+int Target::osscanPerformed(void) {
+	return osscan_flag;
+}
+
+void Target::osscanSetFlag(int flag) {
+	if(osscan_flag == OS_PERF_UNREL)
+		return;
+	else
+		osscan_flag = flag;
+}
+
