@@ -6,17 +6,17 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2004 Insecure.Com LLC. Nmap       *
- * is also a registered trademark of Insecure.Com LLC.  This program is    *
- * free software; you may redistribute and/or modify it under the          *
- * terms of the GNU General Public License as published by the Free        *
- * Software Foundation; Version 2.  This guarantees your right to use,     *
- * modify, and redistribute this software under certain conditions.  If    *
- * you wish to embed Nmap technology into proprietary software, we may be  *
- * willing to sell alternative licenses (contact sales@insecure.com).      *
- * Many security scanner vendors already license Nmap technology such as  *
- * our remote OS fingerprinting database and code, service/version         *
- * detection system, and port scanning code.                               *
+ * The Nmap Security Scanner is (C) 1996-2008 Insecure.Com LLC. Nmap is    *
+ * also a registered trademark of Insecure.Com LLC.  This program is free  *
+ * software; you may redistribute and/or modify it under the terms of the  *
+ * GNU General Public License as published by the Free Software            *
+ * Foundation; Version 2 with the clarifications and exceptions described  *
+ * below.  This guarantees your right to use, modify, and redistribute     *
+ * this software under certain conditions.  If you wish to embed Nmap      *
+ * technology into proprietary software, we sell alternative licenses      *
+ * (contact sales@insecure.com).  Dozens of software vendors already       *
+ * license Nmap technology such as host discovery, port scanning, OS       *
+ * detection, and version detection.                                       *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
@@ -39,7 +39,7 @@
  * These restrictions only apply when you actually redistribute Nmap.  For *
  * example, nothing stops you from writing and selling a proprietary       *
  * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://www.insecure.org/nmap/ to download Nmap.                         *
+ * http://nmap.org to download Nmap.                                       *
  *                                                                         *
  * We don't consider these to be added restrictions on top of the GPL, but *
  * just a clarification of how we interpret "derived works" as it applies  *
@@ -51,10 +51,10 @@
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to many    *
- * security vendors, and generally include a perpetual license as well as  *
- * providing for priority support and updates as well as helping to fund   *
- * the continued development of Nmap technology.  Please email             *
+ * applications and appliances.  These contracts have been sold to dozens  *
+ * of software vendors, and generally include a perpetual license as well  *
+ * as providing for priority support and updates as well as helping to     *
+ * fund the continued development of Nmap technology.  Please email        *
  * sales@insecure.com for further information.                             *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
@@ -78,7 +78,7 @@
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
  * to fyodor@insecure.org for possible incorporation into the main         *
- * distribution.  By sending these changes to Fyodor or one the            *
+ * distribution.  By sending these changes to Fyodor or one of the         *
  * Insecure.Org development mailing lists, it is assumed that you are      *
  * offering Fyodor and Insecure.Com LLC the unlimited, non-exclusive right *
  * to reuse, modify, and relicense the code.  Nmap will always be          *
@@ -98,14 +98,14 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: tcpip.h 2967 2005-12-06 21:15:45Z fyodor $ */
+/* $Id: tcpip.h 7220 2008-04-28 17:53:32Z kris $ */
 
 
 #ifndef TCPIP_H
 #define TCPIP_H
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "nmap_config.h"
 #endif
 
 #include "nbase.h"
@@ -142,13 +142,12 @@ void *realloc();
 #include <sys/param.h> /* Defines MAXHOSTNAMELEN on BSD*/
 #endif
 
-/* Linux uses these defines in netinet/ip.h and netinet/tcp.h to
-   use the correct struct ip and struct tcphdr */
+/* Linux uses these defines in netinet/ip.h to use the correct struct ip */
 #ifndef __FAVOR_BSD
 #define __FAVOR_BSD 1
 #endif
-#ifndef __BSD_SOURCE
-#define __BSD_SOURCE 1
+#ifndef _BSD_SOURCE
+#define _BSD_SOURCE 1
 #endif
 #ifndef __USE_BSD
 #define __USE_BSD 1
@@ -183,22 +182,10 @@ void *realloc();
 #include <netinet/ip.h>
 #define NETINET_IP_H
 #endif
-#ifndef __FAVOR_BSD
-#define __FAVOR_BSD
-#endif
-#ifndef NETINET_TCP_H  /* why the HELL does OpenBSD not do this? */
-#include <netinet/tcp.h>          /*#include <netinet/ip_tcp.h>*/
-#define NETINET_TCP_H
-#endif
-#ifndef NETINET_UDP_H
-#include <netinet/udp.h>
-#define NETINET_UDP_H
-#endif
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 #include <fcntl.h>
-#include <sys/socket.h>
 
 #if TIME_WITH_SYS_TIME
 # include <sys/time.h>
@@ -236,28 +223,9 @@ extern "C" {
 
 typedef enum { devt_ethernet, devt_loopback, devt_p2p, devt_other  } devtype;
 
-#include "nmap_error.h"
-#include "utils.h"
 #include "nmap.h"
 #include "global_structures.h"
 
-
-#ifndef DEBUGGING
-#define DEBUGGING 0
-#endif
-
-#ifndef TCPIP_DEBUGGING
-#define TCPIP_DEBUGGING 0
-#endif
-
-/* Explicit Congestion Notification (rfc 2481/3168) */
-#ifndef TH_ECE
-#define TH_ECE        0x40
-#endif
-#ifndef TH_CWR
-#define TH_CWR        0x80
-#endif
-
 #ifndef INET_ADDRSTRLEN
 #define INET_ADDRSTRLEN 16
 #endif
@@ -388,18 +356,6 @@ struct ip
 
 #endif /* HAVE_STRUCT_IP */
 
-#ifdef LINUX
-typedef struct udphdr_bsd {
-         unsigned short uh_sport;           /* source port */
-         unsigned short uh_dport;           /* destination port */
-         unsigned short uh_ulen;            /* udp length */
-         unsigned short uh_sum;             /* udp checksum */
-} udphdr_bsd;
-#else
- typedef struct udphdr udphdr_bsd;
-#endif 
-
-
 #ifndef HAVE_STRUCT_ICMP
 #define HAVE_STRUCT_ICMP
 /* From Linux /usr/include/netinet/ip_icmp.h GLIBC */
@@ -477,55 +433,10 @@ struct icmp
 };
 #endif /* HAVE_STRUCT_ICMP */
 
-/* Represents a single probe packet, such as a SYN to port 80 or an
-   ICMP netmask request packet. Values are still in network byte order. */
-class IPProbe {
- public:
-  IPProbe();
-  ~IPProbe();
-/* Takes an IP packet and stores _a copy_ of it, in this Probe,
-   adjusting proper header pointers and such */
-  int storePacket(u8 *ippacket, u32 len);
-  u32 packetbuflen; /* Length of the whole packet */
-  u8 *packetbuf; /* The packet itself */
-  struct ip *ipv4; /* IP header of packet */
-  struct icmp *icmp; /* icmp, tcp, and udp are NULL if the packet has no such header */
-  struct tcphdr *tcp;
-  udphdr_bsd *udp;
-
-  u8 af; /* AF_INET or AF_INET6 */
-  /* Resets everything to NULL.  Frees packetbuf if it is filled.  You
-     can reuse a Probe by calling Reset() and then a new
-     storePacket(). */
-  void Reset(); 
- private:
-
-};
-
-/* Handles an *IPv4* Arp probe */
-class ArpProbe {
- public:
-  ArpProbe();
-  ~ArpProbe();
-/* Takes an ARP packet and stores _a copy_ of it, in this Probe,
-   adjusting proper header pointers and such.  Then length better
-   equal 42! */
-  int storePacket(u8 *arppacket, u32 len);
-  u32 packetbuflen; /* Length of the whole packet */
-  u8 *packetbuf; /* The packet itself */
-  struct in_addr *ipquery; /* IP address this ARP seeks */
-  /* Resets everything to NULL.  Frees packetbuf if it is filled.  You
-     can reuse a Probe by calling Reset() and then a new
-     storePacket(). */
-  void Reset(); 
- private:
-
-};
-
- /* This ideally should be a port that isn't in use for any protocol on our machine or on the target */
-#define MAGIC_PORT 49724
-#define TVAL2LONG(X)  X.tv_sec * 1e6 + X.tv_usec
-#define SA struct sockaddr
+/* Some systems might not have this */
+#ifndef IPPROTO_IGMP
+#define IPPROTO_IGMP 2
+#endif
 
 /* Prototypes */
 /* Converts an IP address given in a sockaddr_storage to an IPv4 or
@@ -560,7 +471,7 @@ bool route_dst(const struct sockaddr_storage *const dst, struct route_nfo *rnfo)
    be set to 1 if dst is directly connected on the ifentry network and
    0 if it requires routing.  If nexthop_ip is not NULL, and routing
    is required, the next hop is filled into nexthop_ip.  This function
-   returns false if no apporpiate interface or route was found and
+   returns false if no appropiate interface or route was found and
    true if it succeeds. */
 bool routethrough(const struct sockaddr_storage * const dest, 
 		  struct intf_entry *ifentry, 
@@ -568,19 +479,31 @@ bool routethrough(const struct sockaddr_storage * const dest,
 
 unsigned short in_cksum(u16 *ptr,int nbytes);
 
+unsigned short magic_tcpudp_cksum(const struct in_addr *src,
+				  const struct in_addr *dst,
+				  u8 proto, u16 len, char *hstart);
+
 /* Build and send a raw tcp packet.  If TTL is -1, a partially random
    (but likely large enough) one is chosen */
-int send_tcp_raw( int sd, struct eth_nfo *eth, const struct in_addr *source, 
-		  const struct in_addr *victim, int ttl, 
-		  u16 sport, u16 dport, u32 seq, u32 ack, u8 flags,
-		  u16 window, u8 *options, int optlen, char *data, 
-		  u16 datalen);
-int send_udp_raw( int sd, struct eth_nfo *eth, struct in_addr *source, 
-		  const struct in_addr *victim, int ttl, u16 sport, 
-		  u16 dport, u16 ipid, char *data, u16 datalen);
+int send_tcp_raw( int sd, struct eth_nfo *eth,
+		  const struct in_addr *source, const struct in_addr *victim, 
+		  int ttl, bool df,
+		  u8* ipopt, int ipoptlen,
+		  u16 sport, u16 dport,
+		  u32 seq, u32 ack, u8 reserved, u8 flags, u16 window, u16 urp,
+		  u8 *options, int optlen,
+		  char *data, u16 datalen);
+int send_udp_raw( int sd, struct eth_nfo *eth,
+		  struct in_addr *source, const struct in_addr *victim,
+		  int ttl, u16 ipid,
+		  u8* ipopt, int ipoptlen,
+		  u16 sport, u16 dport,
+		  char *data, u16 datalen);
 
-int send_ip_raw( int sd, struct eth_nfo *eth, struct in_addr *source, 
-		 const struct in_addr *victim, int ttl, u8 proto, 
+int send_ip_raw( int sd, struct eth_nfo *eth,
+		 struct in_addr *source, const struct in_addr *victim,
+		 u8 proto, int ttl,
+		 u8* ipopt, int ipoptlen,
 		 char *data, u16 datalen);
 
 /* Builds a TCP packet (including an IP header) by packing the fields
@@ -589,11 +512,14 @@ int send_ip_raw( int sd, struct eth_nfo *eth, struct in_addr *source,
    actually sent by this function.  Caller must delete the buffer when
    finished with the packet.  The packet length is returned in
    packetlen, which must be a valid int pointer. */
-u8 *build_tcp_raw(const struct in_addr *source, 
-		  const struct in_addr *victim, int ttl, 
-		  u16 ipid, u16 sport, u16 dport, u32 seq, u32 ack, u8 flags,
-		  u16 window, u8 *options, int optlen, char *data, 
-		  u16 datalen, u32 *packetlen);
+u8 *build_tcp_raw(const struct in_addr *source, const struct in_addr *victim,
+		  int ttl, u16 ipid, u8 tos, bool df,
+		  u8* ipopt, int ipoptlen,
+		  u16 sport, u16 dport,
+		  u32 seq, u32 ack, u8 reserved, u8 flags, u16 window, u16 urp,
+		  u8 *options, int optlen,
+		  char *data, u16 datalen,
+		  u32 *packetlen);
 
 /* Builds a UDP packet (including an IP header) by packing the fields
    with the given information.  It allocates a new buffer to store the
@@ -602,18 +528,38 @@ u8 *build_tcp_raw(const struct in_addr *source,
    finished with the packet.  The packet length is returned in
    packetlen, which must be a valid int pointer. */
 u8 *build_udp_raw(struct in_addr *source, const struct in_addr *victim,
- 		  int ttl, u16 sport, u16 dport, u16 ipid, char *data, 
-		  u16 datalen, u32 *packetlen);
+ 		  int ttl, u16 ipid, u8 tos, bool df,
+		  u8* ipopt, int ipoptlen,
+ 		  u16 sport, u16 dport, 
+ 		  char *data, u16 datalen,
+ 		  u32 *packetlen);
 
-/* Builds an ICMP packet (including an IP header) by packing the fields
+/* Builds an ICMP packet (including an IP header) by packing the
+   fields with the given information.  It allocates a new buffer to
+   store the packet contents, and then returns that buffer.  The
+   packet is not actually sent by this function.  Caller must delete
+   the buffer when finished with the packet.  The packet length is
+   returned in packetlen, which must be a valid int pointer. The
+   id/seq will be converted to network byte order (if it differs from
+   HBO) */
+u8 *build_icmp_raw(const struct in_addr *source, const struct in_addr *victim, 
+		   int ttl, u16 ipid, u8 tos, bool df,
+		   u8* ipopt, int ipoptlen,
+		   u16 seq, unsigned short id, u8 ptype, u8 pcode,
+		   char *data, u16 datalen, u32 *packetlen);
+
+/* Builds an IGMP packet (including an IP header) by packing the fields
    with the given information.  It allocates a new buffer to store the
    packet contents, and then returns that buffer.  The packet is not
    actually sent by this function.  Caller must delete the buffer when
-   finished with the packet.  The packet length is returned in
-   packetlen, which must be a valid int pointer. */
-u8 *build_icmp_raw(const struct in_addr *source, const struct in_addr *victim, 
-		   int ttl, u16 ipid, u16 seq, unsigned short id, u8 ptype, 
-		   u8 pcode, char *data, u16 datalen, u32 *packetlen);
+   finished with the packet.  The packet length is returned in packetlen,
+   which must be a valid int pointer.
+ */
+u8 *build_igmp_raw(const struct in_addr *source, const struct in_addr *victim, 
+		   int ttl, u16 ipid, u8 tos, bool df,
+		   u8* ipopt, int ipoptlen,
+		   u8 ptype, u8 pcode,
+		   char *data, u16 datalen, u32 *packetlen);
 
 /* Builds an IP packet (including an IP header) by packing the fields
    with the given information.  It allocates a new buffer to store the
@@ -622,49 +568,72 @@ u8 *build_icmp_raw(const struct in_addr *source, const struct in_addr *victim,
    finished with the packet.  The packet length is returned in
    packetlen, which must be a valid int pointer. */
 u8 *build_ip_raw(const struct in_addr *source, const struct in_addr *victim, 
-		 int ttl, u8 proto, u16 ipid, char *data, u16 datalen, 
+		 u8 proto,
+		 int ttl, u16 ipid, u8 tos, bool df,
+		 u8* ipopt, int ipoptlen,
+		 char *data, u16 datalen, 
 		 u32 *packetlen);
 
 /* Send a pre-built IPv4 packet */
 int send_ip_packet(int sd, struct eth_nfo *eth, u8 *packet, 
 		   unsigned int packetlen);
 
-/* Create and send all fragments of the pre-built packet */
-/* mtu = MTU - ipv4_headerlen */
-int send_frag_ip_packet(int sd, struct eth_nfo *eth, u8 *packet, 
-			unsigned int packetlen, unsigned int mtu);
-
 /* Decoy versions of the raw packet sending functions ... */
 int send_tcp_raw_decoys( int sd, struct eth_nfo *eth, 
-			 const struct in_addr *victim, int ttl,
-			 u16 sport, u16 dport, u32 seq, u32 ack, u8 flags,
-			 u16 window, u8 *options, int optlen, char *data,
-			 u16 datalen);
+			 const struct in_addr *victim,
+			 int ttl, bool df, 
+			 u8* ipopt, int ipoptlen,
+			 u16 sport, u16 dport,
+			 u32 seq, u32 ack, u8 reserved, u8 flags, u16 window, u16 urp,
+			 u8 *options, int optlen,
+			 char *data, u16 datalen);
 
 int send_udp_raw_decoys( int sd, struct eth_nfo *eth, 
-			 const struct in_addr *victim, int ttl, 
-			 u16 sport, u16 dport, u16 ipid, char *data, 
-			 u16 datalen);
+			 const struct in_addr *victim,
+			 int ttl, u16 ipid,
+			 u8* ipops, int ip,
+			 u16 sport, u16 dport,
+			 char *data, u16 datalen);
 
 
 /* Calls pcap_open_live and spits out an error (and quits) if the call fails.
    So a valid pcap_t will always be returned. */
 pcap_t *my_pcap_open_live(const char *device, int snaplen, int promisc, 
 			  int to_ms);
+
+// Returns whether the system supports pcap_get_selectable_fd() properly
+bool pcap_selectable_fd_valid();
+
+/* Call this instead of pcap_get_selectable_fd directly (or your code
+   won't compile on Windows).  On systems which don't seem to support
+   the pcap_get_selectable_fd() function properly, returns -1,
+   otherwise simply calls pcap_selectable_fd and returns the
+   results.  If you just want to test whether the function is supported,
+   use pcap_selectable_fd_valid() instead. */
+int my_pcap_get_selectable_fd(pcap_t *p);
+
 // Returns whether the packet receive time value obtaned from libpcap
 // (and thus by readip_pcap()) should be considered valid.  When
 // invalid (Windows and Amiga), readip_pcap returns the time you called it.
 bool pcap_recv_timeval_valid();
 
-/* Returns a buffer of ASCII information about a packet that may look
-   like "TCP 127.0.0.1:50923 > 127.0.0.1:3 S ttl=61 id=39516 iplen=40
-   seq=625950769" or "ICMP PING (0/1) ttl=61 id=39516 iplen=40".
-   Since this is a static buffer, don't use threads or call twice
-   within (say) printf().  And certainly don't try to free() it!  The
-   returned buffer is NUL-terminated */
-const char *ippackethdrinfo(const u8 *packet, u32 len);
-/* Shows the most important fields of an IP packet (including dissecting TCP/UDP headers.  packet should point to the beginning of the IP header  */
-void readippacket(const u8 *packet, int readdata);
+/* Prints stats from a pcap descriptor (number of received and dropped
+   packets). */
+void pcap_print_stats(int logt, pcap_t *pd);
+
+/* A simple function that caches the eth_t from dnet for one device,
+   to avoid opening, closing, and re-opening it thousands of tims.  If
+   you give a different device, this function will close the first
+   one.  Thus this should never be used by programs that need to deal
+   with multiple devices at once.  In addition, you MUST NEVER
+   eth_close() A DEVICE OBTAINED FROM THIS FUNCTION.  Instead, you can
+   call eth_close_cached() to close whichever device (if any) is
+   cached.  Returns NULL if it fails to open the device. */
+eth_t *eth_open_cached(const char *device);
+
+/* See the description for eth_open_cached */
+void eth_close_cached();
+
 /* A simple function I wrote to help in debugging, shows the important fields
    of a TCP packet*/
 int readtcppacket(const u8 *packet, int readdata);
@@ -683,6 +652,7 @@ struct interface_info *getInterfaceByIP(struct sockaddr_storage *ss);
 struct interface_info *getInterfaceByName(char *iname);
 /* Where the above 4 functions get their info */
 struct interface_info *getinterfaces(int *howmany);
+pcap_if_t *getpcapinterfaces();
 
 /* Parse the system routing table, converting each route into a
    sys_route entry.  Returns an array of sys_routes.  numroutes is set
@@ -693,6 +663,8 @@ struct interface_info *getinterfaces(int *howmany);
 struct sys_route *getsysroutes(int *howmany);
 void sethdrinclude(int sd);
 
+void set_ttl(int sd, int ttl);
+
 /* Fill buf (up to buflen -- truncate if necessary but always
    terminate) with a short representation of the packet stats.
    Returns buf.  Aborts if there is a problem. */
@@ -725,12 +697,11 @@ int setTargetMACIfAvailable(Target *target, struct link_header *linkhdr,
 bool setTargetNextHopMAC(Target *target);
 
 int islocalhost(const struct in_addr * const addr);
+int isipprivate(const struct in_addr * const addr);
 int unblock_socket(int sd);
-int Sendto(char *functionname, int sd, const unsigned char *packet, int len, 
-	   unsigned int flags, struct sockaddr *to, int tolen);
 
 // Takes a protocol number like IPPROTO_TCP, IPPROTO_UDP, or
-// IPPROTO_TCP and returns a ascii representation (or "unknown" if it
+// IPPROTO_IP and returns a ascii representation (or "unknown" if it
 // doesn't recognize the number).  If uppercase is true, the returned
 // value will be in all uppercase letters.  You can skip this
 // parameter to use lowercase.
@@ -744,19 +715,6 @@ int get_link_offset(char *device);
 char *readip_pcap(pcap_t *pd, unsigned int *len, long to_usec, 
 		  struct timeval *rcvdtime, struct link_header *linknfo);
 
-/* A trivial functon that maintains a cache of IP to MAC Address
-   entries.  If the command is ARPCACHE_GEt, this func looks for the
-   IPv4 address in ss and fills in the 'mac' parameter and returns
-   true if it is found.  Otherwise (not found), the function returns
-   false.  If the command is ARPCACHE_SET, the function adds an entry
-   with the given ip (ss) and mac address.  An existing entry for the
-   IP ss will be overwritten with the new MAC address.  true is always
-   returned for the set command. */
-#define ARPCACHE_GET 1
-#define ARPCACHE_SET 2
-bool NmapArpCache(int command, struct sockaddr_storage *ss, u8 *mac);
-
-
 /* Attempts to read one IPv4/Ethernet ARP reply packet from the pcap
    descriptor pd.  If it receives one, fills in sendermac (must pass
    in 6 bytes), senderIP, and rcvdtime (can be NULL if you don't care)
@@ -767,19 +725,6 @@ bool NmapArpCache(int command, struct sockaddr_storage *ss, u8 *mac);
 int read_arp_reply_pcap(pcap_t *pd, u8 *sendermac, struct in_addr *senderIP,
 		       long to_usec, struct timeval *rcvdtime);
 
-/* Issues an ARP request for the MAC of targetss (which will be placed
-   in targetmac if obtained) from the source IP (srcip) and source mac
-   (srcmac) given.  "The request is ussued using device dev to the
-   broadcast MAC address.  The transmission is attempted up to 3
-   times.  If none of these elicit a response, false will be returned.
-   If the mac is determined, true is returned. */
-bool doArp(const char *dev, u8 *srcmac, struct sockaddr_storage *srcip, 
-	   struct sockaddr_storage *targetip, u8 *targetmac);
-
-#ifndef HAVE_INET_ATON
-int inet_aton(register const char *, struct in_addr *);
-#endif
-
 /* Examines the given tcp packet and obtains the TCP timestamp option
    information if available.  Note that the CALLER must ensure that
    "tcp" contains a valid header (in particular the th_off must be the
@@ -790,7 +735,7 @@ int inet_aton(register const char *, struct in_addr *);
    parameters (if non-null) are filled with 0.  Remember that the
    correct way to check for errors is to look at the return value
    since a zero ts or echots could possibly be valid. */
-int gettcpopt_ts(struct tcphdr *tcp, u32 *timestamp, u32 *echots);
+int gettcpopt_ts(struct tcp_hdr *tcp, u32 *timestamp, u32 *echots);
 
 /* Maximize the receive buffer of a socket descriptor (up to 500K) */
 void max_rcvbuf(int sd);
@@ -815,20 +760,8 @@ int recvtime(int sd, char *buf, int len, int seconds, int *timedout);
 
 /* Sets a pcap filter function -- makes SOCK_RAW reads easier */
 #ifndef WINIP_H
-typedef int (*PFILTERFN)(const char *packet, unsigned int len); /* 1 to keep */
-void set_pcap_filter(const char *device, pcap_t *pd, char *bpf, ...);
+void set_pcap_filter(const char *device, pcap_t *pd, const char *bpf, ...);
 #endif
 
-/* Just accept everything ... TODO: Need a better approach than this flt_ 
-   stuff */
-int flt_all(const char *packet, unsigned int len);
-int flt_icmptcp(const char *packet, unsigned int len);
-int flt_icmptcp_2port(const char *packet, unsigned int len);
-int flt_icmptcp_5port(const char *packet, unsigned int len);
-
 #endif /*TCPIP_H*/
 
-
-
-
-
